/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: David Bainbridge, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

package org.greenstone.gatherer.util;

import java.io.*;
import java.util.*;
import java.util.zip.*;


public class Unzip 
{

    public static final void copyInputStream(InputStream in, OutputStream out)
	throws IOException
    {
	byte[] buffer = new byte[1024];
	int len;

	while((len = in.read(buffer)) >= 0)
	    out.write(buffer, 0, len);

	in.close();
	out.close();
    }

    static public void main(String[] args) 
    {
	Enumeration entries;
	ZipFile zipFile;

	if(args.length != 2) {
	    System.err.println("Usage: Unzip gsdlhome zipfile");
	    return;
	}

	String gsdl_home = args[0];
	String col_dir = gsdl_home + "collect" + File.separator;

	try {
	    zipFile = new ZipFile(args[1]);

	    entries = zipFile.entries();

	    while(entries.hasMoreElements()) {
		ZipEntry entry = (ZipEntry)entries.nextElement();

		if(entry.isDirectory()) {
		    // Assume directories are stored parents first then children.
		    // System.err.println("Extracting directory: " + entry.getName());
		    // This is not robust, just for demonstration purposes.
		    String dir_name = entry.getName();
		    String full_dir_name = col_dir + dir_name;
		    File dirFile = new File(full_dir_name);
		    dirFile.mkdir();
		}
		else {
		    // System.err.println("Extracting file: " + entry.getName());
		    String file_name = entry.getName();
		    String full_file_name = col_dir + file_name;
		    FileOutputStream fos = new FileOutputStream(full_file_name);
		    BufferedOutputStream bos = new BufferedOutputStream(fos);

		    copyInputStream(zipFile.getInputStream(entry),bos);
		}
	    }

	    zipFile.close();
	} 
	catch (IOException ioe) {
	    System.err.println("Unhandled exception on " + args[1]);
	    ioe.printStackTrace();
	    return;
	}
    }
}
